/*
<moRestartPerturb.h>
Copyright (C) DOLPHIN Project-Team, INRIA Lille - Nord Europe, 2006-2010

Sébastien Verel, Arnaud Liefooghe, Jérémie Humeau

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  ue,
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.
The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

ParadisEO WebSite : http://paradiseo.gforge.inria.fr
Contact: paradiseo-help@lists.gforge.inria.fr
*/

#ifndef _moRestartPerturb_h
#define _moRestartPerturb_h


#include <eoEvalFunc.h>
#include <eoInit.h>
#include <perturb/moPerturbation.h>
#include <memory/moCountMoveMemory.h>

/**
 * Restart Perturbation : restart when maximum number of iteration with no improvement is reached
 */
template< class Neighbor >
class moRestartPerturb : public moPerturbation<Neighbor>, public moCountMoveMemory<Neighbor> {

public:
    typedef typename Neighbor::EOT EOT;

    /**
     * Constructor
     * @param _initializer an initializer of solution
     * @param _fullEval a full evaluation function
     * @param _threshold maximum number of iteration with no improvement
     */
    moRestartPerturb(eoInit<EOT>& _initializer, eoEvalFunc<EOT>& _fullEval, unsigned int _threshold):initializer(_initializer), fullEval(_fullEval), threshold(_threshold) {}

    /**
     * Apply restart when necessary
     * @param _solution to restart
     * @return true
     */
    bool operator()(EOT& _solution) {
        if ((*this).getCounter()>= threshold) {
            initializer(_solution);
            fullEval(_solution);
            (*this).initCounter();
        }
        return true;
    }

private:
    eoInit<EOT>& initializer;
    eoEvalFunc<EOT>& fullEval;
    unsigned int threshold;
};


#endif
